@@ -362,4 +362,129 @@
    nil
    ])
 
+(defconst ada-elisp-keyword-table-raw
+  '(
+   ("abs" . ABS)
+   ("accept" . ACCEPT)
+   ("abort" . ABORT)
+   ("abstract" . ABSTRACT)
+   ("access" . ACCESS)
+   ("aliased" . ALIASED)
+   ("all" . ALL)
+   ("and" . AND)
+   ("array" . ARRAY)
+   ("at" . AT)
+   ("begin" . BEGIN)
+   ("body" . BODY)
+   ("case" . CASE)
+   ("constant" . CONSTANT)
+   ("declare" . DECLARE)
+   ("delay" . DELAY)
+   ("delta" . DELTA)
+   ("digits" . DIGITS)
+   ("do" . DO)
+   ("else" . ELSE)
+   ("elsif" . ELSIF)
+   ("end" . END)
+   ("entry" . ENTRY)
+   ("exception" . EXCEPTION)
+   ("exit" . EXIT)
+   ("for" . FOR)
+   ("function" . FUNCTION)
+   ("generic" . GENERIC)
+   ("goto" . GOTO)
+   ("if" . IF)
+   ("in" . IN)
+   ("interface" . INTERFACE)
+   ("is" . IS)
+   ("limited" . LIMITED)
+   ("loop" . LOOP)
+   ("mod" . MOD)
+   ("new" . NEW)
+   ("not" . NOT)
+   ("null" . NULL)
+   ("of" . OF)
+   ("or" . OR)
+   ("others" . OTHERS)
+   ("out" . OUT)
+   ("overriding" . OVERRIDING)
+   ("package" . PACKAGE)
+   ("pragma" . PRAGMA)
+   ("private" . PRIVATE)
+   ("procedure" . PROCEDURE)
+   ("protected" . PROTECTED)
+   ("raise" . RAISE)
+   ("range" . RANGE)
+   ("record" . RECORD)
+   ("rem" . REM)
+   ("renames" . RENAMES)
+   ("requeue" . REQUEUE)
+   ("return" . RETURN)
+   ("reverse" . REVERSE)
+   ("separate" . SEPARATE)
+   ("select" . SELECT)
+   ("some" . SOME)
+   ("subtype" . SUBTYPE)
+   ("synchronized" . SYNCHRONIZED)
+   ("tagged" . TAGGED)
+   ("task" . TASK)
+   ("terminate" . TERMINATE)
+   ("then" . THEN)
+   ("type" . TYPE)
+   ("until" . UNTIL)
+   ("use" . USE)
+   ("when" . WHEN)
+   ("while" . WHILE)
+   ("with" . WITH)
+   ("xor" . XOR)
+   ))
+
+(defconst ada-elisp-token-table-raw
+  '(
+   ("left-paren"
+    (LEFT_PAREN . "(")
+    )
+   ("right-paren"
+    (RIGHT_PAREN . ")")
+    )
+   ("punctuation"
+    (AMPERSAND . "&")
+    (BAR . "|")
+    (BOX . "<>")
+    (COLON . ":")
+    (COLON_EQUAL . ":=")
+    (COMMA . ",")
+    (DOT . ".")
+    (DOT_DOT . "..")
+    (EQUAL . "=")
+    (EQUAL_GREATER . "=>")
+    (GREATER . ">")
+    (GREATER_EQUAL . ">=")
+    (GREATER_GREATER . ">>")
+    (LESS . "<")
+    (LESS_EQUAL . "<=")
+    (LESS_LESS . "<<")
+    (MINUS . "-")
+    (PLUS . "+")
+    (SEMICOLON . ";")
+    (SLASH . "/")
+    (SLASH_EQUAL . "/=")
+    (STAR . "*")
+    (STAR_STAR . "**")
+    (TICK_1 . "'")
+    )
+   ("number"
+    (NUMERIC_LITERAL ada-wisi-number-p)
+    )
+   ("symbol"
+    (IDENTIFIER . "")
+    )
+   ("string-double"
+    (STRING_LITERAL . "")
+    )
+   ("string-single"
+    (CHARACTER_LITERAL . "")
+    )
+   ))
+
 (provide 'ada-process)
