@@ -23,6 +23,7 @@
    NEW_LINE
    COMMENT
    ABSTRACT
+   AT
    AGGREGATE
    CASE
    CONFIGURATION
@@ -54,6 +55,7 @@
    QUOTE
    SEMICOLON
    VERTICAL_BAR
+   NUMERIC_LITERAL
    IDENTIFIER
    STRING_LITERAL
    Wisi_EOI
@@ -99,4 +101,56 @@
    nil
    ])
 
+(defconst gpr-elisp-keyword-table-raw
+  '(
+   ("abstract" . ABSTRACT)
+   ("at" . AT)
+   ("aggregate" . AGGREGATE)
+   ("case" . CASE)
+   ("configuration" . CONFIGURATION)
+   ("end" . END)
+   ("extends" . EXTENDS)
+   ("external" . EXTERNAL)
+   ("external_as_list" . EXTERNAL_AS_LIST)
+   ("for" . FOR)
+   ("is" . IS)
+   ("(" . LEFT_PAREN)
+   ("library" . LIBRARY)
+   ("null" . NULL)
+   ("others" . OTHERS)
+   ("package" . PACKAGE)
+   ("project" . PROJECT)
+   ("renames" . RENAMES)
+   (")" . RIGHT_PAREN)
+   ("standard" . STANDARD)
+   ("type" . TYPE)
+   ("use" . USE)
+   ("when" . WHEN)
+   ("with" . WITH)
+   ))
+
+(defconst gpr-elisp-token-table-raw
+  '(
+   ("punctuation"
+    (AMPERSAND . "&")
+    (COLON . ":")
+    (COLON_EQUALS . ":=")
+    (COMMA . ",")
+    (DOT . ".")
+    (EQUAL_GREATER . "=>")
+    (QUOTE . "'")
+    (SEMICOLON . ";")
+    (VERTICAL_BAR . "|")
+    )
+   ("number"
+    (NUMERIC_LITERAL ada-wisi-number-p)
+    )
+   ("symbol"
+    (IDENTIFIER . "")
+    )
+   ("string-double"
+    (STRING_LITERAL . "")
+    )
+   ))
+
 (provide 'gpr-process)
